Attribute VB_Name = "PhCommon22"

Option Explicit

   ' The core Phidget class deals with functionality common to all
   ' Phidgets, such as opening and closing them, or setting Attach,
   ' Detach and Event handlers.
   '
   ' This class is also used to specify the associations between the
   ' Phidget software objects and their corresponding physical
   ' devices, and makes it possible to determine which Phidget is
   ' which in cases where it might otherwise be ambiguous.
   ' This class contains various functions such as Release, Retain,
   ' and getParent designed to be used with the Phidget Manager.
   ' These specialized functions may be safely ignored if your
   ' application does not require a Manager.  You can check the
   ' Manager API for more information.
   
   ' =======================================================================
   ' WORK IN PROGRESS !!!     === Contains ERRORS ===
   '
   ' Translation from Phidget22.h C++ code, to Visual Basic 6 code.
   '
   ' Most function comments contain the orginal Phidget22 C++ code.
   '
   ' =======================================================================
   ' Visual Basic V6.0 type-declaration characters
   ' $=STRING    %=INTEGER int16    &=LONG int32    !=SINGLE flo32
   ' #=DOUBLE flo64   @=Currency 64bit +/-922,337,203,685,477.5807 fixed "."
   ' =======================================================================
   '     VB Type      C++ Type
   '     --------------------------
   '     Byte         unsigned char
   '     Integer%     short
   '     Long&        int,uint32,long (unsigned int requires special handling)
   '     Single!      float
   '     Double#      double
   '     Currency@    int64
   ' =============================
   
 
   ' =======================================================================
   Public Declare Sub Sleep _
      Lib "kernel32" ( _
      ByVal dwMilliseconds&)
   
   Public Declare Function StrLen& _
      Lib "kernel32.dll" _
      Alias "lstrlenA" ( _
      ByVal strPtr&)

   Public Declare Function SysAllocStringByteLen$ _
      Lib "oleaut32.dll" ( _
      Optional ByVal pszStrPtr&, _
      Optional ByVal strLength&)
  



'===========================================================================
' PHIDGET COMMON MODULE FUNCTIONS
'===========================================================================
' Contains various functions such as Release, Retain, and getParent designed
' to be used with the Phidget Manager.  These specialized functions may be
' safely ignored if your application does not require a Manager.
   
   ' =======================================================================
   ' Functions:
   ' =======================================================================

   ' =======================================================================
   ' Phidget_getErrorDescription(
   '     PhidgetReturnCode errorCode,
   '     const char **errorString
   ' )
   ' Prameters:
   '     errorCode: The error code
   '     errorString: The error string
   ' Return: The Phidget return code
   ' Gets the string describing the given Phidget return code
   ' NOTE: To get the error String from the error Code, use
   '       errStr$ = PhGetErrorDescription&(errCode&)
   '       in place of this function.
   ' OR:   After calling this function use:
   '       ErrStr$ = SysAllocStringByteLen$(errStrPtr&, StrLen&(errStrPtr&))
   '
   Public Declare Function Phidget_getErrorDescription& _
      Lib "Phidget22.dll" ( _
      ByVal errCode&, _
      ByRef errStrPtr&)
      
   ' =======================================================================
   ' Phidget_finalize(
   '     int32_t  flags
   ' )
   ' Prameters:
   '     flags: Reserved for future use. Pass 0.
   ' Return: The Phidget return code
   ' Release memory and threads used by the Phidget library.  Should be
   ' called prior to unloading the library from the address space.
   ' This function is intended for use in special cases where it is
   ' desired for the Phidget library to be unloaded before a program's
   ' termination.
   '
   Public Declare Function PhFinalize& _
      Lib "Phidget22.dll" _
      Alias "Phidget_finalize" ( _
      ByVal flags&)           ' unsigned integer

   ' =======================================================================
   ' Phidget_getLibraryVersion(
   '     const char **libraryVersion
   ' )
   ' Prameters:
   '     libraryVersion: The Phidget library version.
   ' Return: The Phidget return code
   '
   ' Gets the version of the Phidget library being used by the program.
   ' NOTE: After calling this function use:
   '       Str$ = SysAllocStringByteLen$(...StrPtr&, StrLen&(...StrPtr&))
   '
    Public Declare Function PhGetLibraryVersion& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getLibraryVersion" ( _
      ByRef libraryVersionStrPtr&)

   ' =======================================================================
   ' Phidget_release(
   '     PhidgetHandle *phid
   ' )
   ' Prameters:
   '     phid: The Phidget handle to release
   ' Return: The Phidget return code
   ' Releases an existing reference to a Phidget handle. This may cause the
   ' handle's resources to be freed; therefore, the handle should not be
   ' referred to again after it is released.  Phidget_release() must not be
   ' called on handles that have not had Phidget_retain() called on them.
   '
   Public Declare Function PhRelease& _
      Lib "Phidget22.dll" _
      Alias "Phidget_release" ( _
      ByRef handle&)

   ' =======================================================================
   ' Phidget_retain(
   '     PhidgetHandle phid
   ' )
   ' Prameters:
   '     phid: The Phidget handle to retain
   ' Return: The Phidget return code
   ' Creates a additional reference to a Phidget handle. Phidget_retain()
   ' is used to keep a handle from a Manager event alive after the event
   ' method returns.
   '
   Public Declare Function PhRetain& _
      Lib "Phidget22.dll" _
      Alias "Phidget_retain" ( _
      ByVal handle&)
   
   ' =======================================================================
   ' Phidget_getAttached(
   '     PhidgetHandle ch,
   '     int *attached
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     attached: True if the channel is attached
   ' Return: The Phidget return code
   ' Gets the attached status of channel. A Phidget is attached after it
   ' has been opened and the Phidget library finds and connects to the
   ' corresponding hardware device.  Most API calls are only valid on
   ' attached Phidgets.
   '
   Public Declare Function PhGetAttached& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getAttached" ( _
      ByVal handle&, _
      ByRef attached&)

   ' =======================================================================
   ' Phidget_getChannel(
   '     PhidgetHandle ch,
   '     int *channel
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     channel: The channel index
   ' Return: The Phidget return code
   ' Gets the channel index of the channel on the device.
   '
   Public Declare Function PhGetChannel& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getChannel" ( _
      ByVal handle&, _
      ByRef channel&)

   ' =======================================================================
   ' Phidget_setChannel(
   '     PhidgetHandle ch,
   '     int channel
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     channel: The channel index
   ' Return: The Phidget return code
   ' Specifies the channel index to be opened. The default channel is 0.
   ' Set to PHIDGET_CHANNEL_ANY to open any channel on the specified device.
   '
   Public Declare Function PhSetChannel& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setChannel" ( _
      ByVal handle&, _
      ByVal channel&)

   ' =======================================================================
   ' Phidget_getChannelClass(
   '     PhidgetHandle ch,
   '     Phidget_ChannelClass *channelClass
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     channelClass: The channel class
   ' Return: The Phidget return code
   ' Gets the channel class of the channel.
   Public Declare Function PhGetChannelClass& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getChannelClass" ( _
      ByVal handle&, _
      ByRef channelClass&)

   ' =======================================================================
   ' Phidget_getChannelClassName(
   '     PhidgetHandle ch,
   '     const char **channelClassName
   ' )
   ' Prameters:
   ' Gets the name of the channel class the channel belongs to.
   '     ch: The channel handle
   '     channelClassName: The name of the channel's class
   ' Return: The Phidget return code
   '     Constant: PHIDCHCLASS_HUB
   ' NOTE: After calling this function use:
   '       Str$ = SysAllocStringByteLen$(...StrPtr&, StrLen&(...StrPtr&))
   '
   Public Declare Function PhGetChannelClassName& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getChannelClassName" ( _
      ByVal handle&, _
      ByRef serverHostnameStrPtr&)

   ' =======================================================================
   ' Phidget_getChannelName(
   '     PhidgetHandle ch,
   '     const char **channelName
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     channelName: The channel's name
   ' Return: The Phidget return code
   ' Gets the channel's name. This name serves as a description of the
   ' specific nature of the channel.
   ' NOTE: After calling this function use:
   '       Str$ = SysAllocStringByteLen$(...StrPtr&, StrLen&(...StrPtr&))
   '
   Public Declare Function PhGetChannelName& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getChannelName" ( _
      ByVal handle&, _
      ByRef channelNameStrPtr&)

   ' =======================================================================
   ' Phidget_getChannelSubclass(
   '     PhidgetHandle ch,
   '     Phidget_ChannelSubclass *channelSubclass
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     channelSubclass: The channel's subclass
   ' Return: The Phidget return code
   ' Gets the subclass for this channel. Allows for identifying channels
   ' with specific characteristics without needing to know the exact
   ' device and channel index.
   '
   Public Declare Function PhGetChannelSubclass& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getChannelSubclass" ( _
      ByVal handle&, _
      ByRef channelSubclass&)

   ' =======================================================================
   ' Phidget_close(
   '     PhidgetHandle ch
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     Constant: PHIDCHSUBCLASS_NONE
   ' Return: The Phidget return code
   ' Closes a Phidget channel that has been opened. Phidget_close() will
   ' release the channel on the Phidget device, and should be called
   ' prior to delete.
   '
   Public Declare Function PhClose& _
      Lib "Phidget22.dll" _
      Alias "Phidget_close" ( _
      ByVal handle&)

   ' =======================================================================
   ' Phidget_getDataInterval(
   '     PhidgetHandle ch,
   '     uint32_t *dataInterval
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     dataInterval: The data interval of the channel
   ' Return: The Phidget return code
   ' Gets the data interval for any channel that supports one.
   ' The DataInterval is the time that must elapse before the channel
   ' will fire another data event.  The data interval is bounded by
   ' MinDataInterval and MaxDataInterval.  The timing between data events
   ' can also be affected by the specific change trigger for the channel.
   '
   Public Declare Function PhGetDataInterval& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getDataInterval" ( _
      ByVal handle&, _
      ByRef dataInterval&)    ' unsigned integer
      
   ' =======================================================================
   ' Phidget_setDataInterval(
   '     PhidgetHandle ch,
   '     uint32_t dataInterval
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     dataInterval: The data interval of the channel
   ' Return: The Phidget return code
   ' Sets the data interval for any channel that supports one.
   ' The DataInterval is the time that must elapse before the channel
   ' will fire another data event.  The data interval is bounded by
   ' MinDataInterval and MaxDataInterval.  The timing between data events
   ' can also be affected by the specific change trigger for the channel.
   '
   Public Declare Function PhSetDataInterval& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setDataInterval" ( _
      ByVal handle&, _
      ByVal dataInterval&)    ' unsigned integer
   
   ' =======================================================================
   ' Phidget_getDeviceChannelCount(
   '     PhidgetHandle ch,
   '     Phidget_ChannelClass cls,
   '     uint32_t *count
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     cls: The Channel Class
   '     count: The Channel Count
   ' Return: The Phidget return code
   ' Gets the number of channels of the specified channel class on the
   ' device.  Pass PHIDCHCLASS_NOTHING to get the total number of channels.
   '
   Public Declare Function PhGetDeviceChannelCount& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getDeviceChannelCount" ( _
      ByVal handle&, _
      ByRef count&)           ' unsigned integer

   ' =======================================================================
   ' Phidget_getDeviceClass(
   '     PhidgetHandle ch,
   '     Phidget_DeviceClass *deviceClass
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     deviceClass: The class of the device the channel is a part of.
   ' Return: The Phidget return code
   ' Gets the device class for the Phidget which this channel is a part of.
   '
   Public Declare Function PhGetDeviceClass& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getDeviceClass" ( _
      ByVal handle&, _
      ByRef deviceClass&)

   ' =======================================================================
   ' Phidget_getDeviceClassName(
   '     PhidgetHandle ch,
   '     const char **deviceClassName
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     deviceClassName: The class name of the device the channel is a part of.
   ' Return: The Phidget return code
   '     Constant: PHIDCLASS_HUB
   ' Gets the name of the device class for the Phidget which this channel
   ' is a part of.
   ' NOTE: After calling this function use:
   '       Str$ = SysAllocStringByteLen$(...StrPtr&, StrLen&(...StrPtr&))
   '
   Public Declare Function PhGetDeviceClassName& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getDeviceClassName" ( _
      ByVal handle&, _
      ByRef deviceClassNameStrPtr&)

   ' =======================================================================
   ' Phidget_getDeviceID(
   '     PhidgetHandle ch,
   '     Phidget_DeviceID *deviceID
   ' )
   ' Prameters:
   ' Gets the DeviceID for the Phidget which this channel is a part of.
   '     ch: The channel handle
   '     deviceID: The device id of the device the channel is a part of
   ' Return: The Phidget return code
   '
   Public Declare Function PhGetDeviceID& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getDeviceID" ( _
      ByVal handle&, _
      ByRef deviceId&)

   ' =======================================================================
   ' Phidget_getDeviceLabel(
   '     PhidgetHandle ch,
   '     const char **deviceLabel
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     deviceLabel: The device label
   ' Return: The Phidget return code
   ' Gets the label of the Phidget which this channel is a part of. A
   ' device label is a custom string used to more easily identify a
   ' Phidget. Labels are written to a Phidget using Phidget_writeDeviceLabel(),
   ' or by right-clicking the device and setting a label in the Phidget Control
   ' Panel for Windows.
   ' See using a label for more information about how to use labels with Phidgets.
   ' NOTE: After calling this function use:
   '       Str$ = SysAllocStringByteLen$(deviceLabelStrPtr&, StrLen&(deviceLabelStrPtr&))
   '
   Public Declare Function PhGetDeviceLabel& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getDeviceLabel" ( _
      ByVal handle&, _
      ByRef deviceLabelStrPtr&)

   ' =======================================================================
   ' Phidget_setDeviceLabel(
   '     PhidgetHandle ch,
   '     const char *deviceLabel
   ' )
   ' Prameters:
   '     Constant: PHIDID_HUB0000
   '     ch: The channel handle
   '     DeviceLabel: The device label
   ' Return: The Phidget return code
   ' Specifies the label of the Phidget to be opened.  Leave un-set to open
   ' any label.  A device label is a custom string used to more easily
   ' identify a Phidget.  Labels are written to a Phidget using
   ' Phidget_writeDeviceLabel(), or by right-clicking the device
   ' and setting a label in the Phidget Control Panel for Windows.
   ' See using a label for more information about how to use labels with Phidgets.
   ' Example:  stat& = PhSetDeviceLabel&(handle&, StrPtr(deviceLabel$))
   '
   ' Note! This writes data to the Flash memory that has a limited number
   '       of writes and is not supported on very old Phidgets.
   '       I'm not sure it works with VB6.
   '
   Public Declare Function PhSetDeviceLabel& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setDeviceLabel" ( _
      ByVal handle&, _
      ByRef deviceLabel$)
   
   ' =======================================================================
   ' Phidget_getDeviceName(
   '     PhidgetHandle ch,
   '     const char **deviceName
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     deviceName: The name of the device the channel is a part of
   ' Return: The Phidget return code
   ' Gets the name of the Phidget which this channel is a part of.
   ' NOTE: After calling this function use:
   '       Str$ = SysAllocStringByteLen$(...StrPtr&, StrLen&(...StrPtr&))
   '
   Public Declare Function PhGetDeviceName& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getDeviceName" ( _
      ByVal handle&, _
      ByRef deviceNameStrPtr&)

   ' =======================================================================
   ' Phidget_getDeviceSerialNumber(
   '     PhidgetHandle ch,
   '     int32_t *deviceSerialNumber
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     deviceSerialNumber: The device serial number
   ' Return: The Phidget return code
   ' Gets the serial number of the Phidget which this channel is a part of.
   ' If the channel is part of a VINT device, this will be the serial
   ' number of the VINT Hub the device is attached to.
   '
   Public Declare Function PhGetDeviceSerialNumber& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getDeviceSerialNumber" ( _
      ByVal handle&, _
      ByRef deviceSerialNumber&)

   ' =======================================================================
   ' Phidget_setDeviceSerialNumber(
   '     PhidgetHandle ch,
   '     int32_t deviceSerialNumber
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     deviceSerialNumber: The device serial number
   ' Return: The Phidget return code
   ' Specifies the serial number of the Phidget to be opened. Leave un-set,
   ' or set to PHIDGET_SERIALNUMBER_ANY to open any serial number.  If the
   ' channel is part of a VINT device, this will be the serial number of
   ' the VINT Hub the device is attached to.
   '
   Public Declare Function PhSetDeviceSerialNumber& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setDeviceSerialNumber" ( _
      ByVal handle&, _
      ByVal deviceSerialNumber&)

   ' =======================================================================
   ' Phidget_getDeviceSKU(
   '     PhidgetHandle ch,
   '     const char **deviceSKU
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     deviceSKU: The SKU of the device the channel is a part of
   ' Return: The Phidget return code
   ' Gets the SKU (part number) of the Phidget which this channel is a part of.
   ' NOTE: After calling this function use:
   '       Str$ = SysAllocStringByteLen$(...StrPtr&, StrLen&(...StrPtr&))
   '
   Public Declare Function PhGetDeviceSKU& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getDeviceSKU" ( _
      ByVal handle&, _
      ByRef deviceSkuStrPtr&)

   ' =======================================================================
   ' Phidget_getDeviceVersion(
   '     PhidgetHandle ch,
   '     int *deviceVersion
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     deviceVersion: The version of the device the channel is a part of
   ' Return: The Phidget return code
   ' Gets the firmware version of the Phidget which this channel is a part of.
   '
   Public Declare Function PhGetDeviceVersion& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getDeviceVersion" ( _
      ByVal handle&, _
      ByRef deviceVersion&)

   ' =======================================================================
   ' Phidget_getHub(
   '     PhidgetHandle ch,
   '     PhidgetHandle *hub
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     hub: The hub the channels device is attached to
   ' Return: The Phidget return code
   ' Gets the hub that this channel is attached to. Must be freed with
   ' Phidget_release()
   '
   Public Declare Function PhGetHub& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getHub" ( _
      ByVal handle&, _
      ByRef hub&)

   ' =======================================================================
   ' Phidget_getHubPort(
   '     PhidgetHandle ch,
   '     int *hubPort
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     hubPort: The hub port index
   ' Return: The Phidget return code
   ' Gets the hub port index of the VINT Hub port that the channel is attached to.
   '
   Public Declare Function PhGetHubPort& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getHubPort" ( _
      ByVal handle&, _
      ByRef hubPort&)

   ' =======================================================================
   ' Phidget_setHubPort(
   '     PhidgetHandle ch,
   '     int hubPort
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     hubPort: The hub port index
   ' Return: The Phidget return code
   ' Specifies the hub port index of the VINT Hub port to open this channel
   ' on.  Leave un-set, or set to PHIDGET_HUBPORT_ANY to open the channel on
   ' any VINT Hub port
   Public Declare Function PhSetHubPort& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setHubPort" ( _
      ByVal handle&, _
      ByVal hubPort&)

   ' =======================================================================
   ' Phidget_getHubPortCount(
   '     PhidgetHandle ch,
   '     int *hubPortCount
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     hubPortCount: The number of ports on the VINT Hub
   ' Return: The Phidget return code
   ' Gets the number of VINT ports present on the VINT Hub that the channel
   ' is attached to.
   '
   Public Declare Function PhGetHubPortCount& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getHubPortCount" ( _
      ByVal handle&, _
      ByRef hubPortCount&)

   ' =======================================================================
   ' Phidget_getIsChannel(
   '     PhidgetHandle ch,
   '     int *isChannel
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     isChannel: True if the handle is for a channel.
   ' Return: The Phidget return code
   ' Returns true if the PhidgetHandle is for a channel. Mostly for use
   ' alongside Phidget_getParent() to distinguish channel handles.
   ' from device handles.
   '
   Public Declare Function PhGetIsChannel& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getIsChannel" ( _
      ByVal handle&, _
      ByRef isChannel&)

   ' =======================================================================
   ' Phidget_getIsHubPortDevice(
   '     PhidgetHandle ch,
   '     int *isHubPortDevice
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     isHubPortDevice: The hub port mode (True if the channel is a hub port itself)
   ' Return: The Phidget return code
   ' Gets whether this channel is a VINT Hub port channel, or part of a VINT
   ' device attached to a hub port.
   '
   Public Declare Function PhGetIsHubPortDevice& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getIsHubPortDevice" ( _
      ByVal handle&, _
      ByRef isHubPortDevice&)

   ' =======================================================================
   ' Phidget_setIsHubPortDevice(
   '     PhidgetHandle ch,
   '     int isHubPortDevice
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     isHubPortDevice: The hub port mode (True if channel is a hub port itself)
   ' Return: The Phidget return code
   ' Specifies whether this channel should be opened on a VINT Hub port
   ' directly, or on a VINT device attached to a hub port.
   '
   Public Declare Function PhSetIsHubPortDevice& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setIsHubPortDevice" ( _
      ByVal handle&, _
      ByVal isHubPortDevice&)

   ' =======================================================================
   ' Phidget_getIsLocal(
   '     PhidgetHandle ch,
   '     int *isLocal
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     isLocal: True if the channel is attached to a local device
   ' Return: The Phidget return code
   ' Returns true when this channel is attached directly on the local
   ' machine, or false otherwise.
   '
   Public Declare Function PhGetIsLocal& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getIsLocal" ( _
      ByVal handle&, _
      ByRef isLocal&)

   ' =======================================================================
   ' Phidget_setIsLocal(
   '     PhidgetHandle ch,
   '     int isLocal
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     isLocal: True if the channel is attached to a local device
   ' Return: The Phidget return code
   ' Set to True if the channel is to be opened locally, and not over a
   ' network.  If both this and IsRemote are set to False (the default),
   ' the channel will be opened either locally or remotely, on whichever
   ' matching channel is found first.
   '
   Public Declare Function PhSetIsLocal& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setIsLocal" ( _
      ByVal handle&, _
      ByVal isLocal&)

   ' =======================================================================
   ' Phidget_getIsRemote(
   '     PhidgetHandle ch,
   '     int *isRemote
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     isRemote: True if the channel is attached to a network device
   ' Return: The Phidget return code
   ' Returns true when this channel is attached via a Phidget network
   ' server, or false otherwise.
   '
   Public Declare Function PhGetIsRemote& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getIsRemote" ( _
      ByVal handle&, _
      ByRef isRemote&)

   ' =======================================================================
   ' Phidget_setIsRemote(
   '     PhidgetHandle ch,
   '     int isRemote
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     isRemote: True if the channel is attached to a network device
   ' Return: The Phidget return code
   ' Set to True if the channel is to be opened remotely, rather than
   ' locally.  If both this and IsLocal are set to False (the default),
   ' the channel will be opened either locally or remotely, on whichever
   ' matching channel is found first.
   '
   Public Declare Function PhSetisRemote& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setIsRemote" ( _
      ByVal handle&, _
      ByVal isRemote&)

   ' =======================================================================
   ' Phidget_open(
   '     PhidgetHandle ch
   ' )
   ' Prameters:
   '     ch: The channel handle
   ' Return: The Phidget return code
   ' Opens the Phidget channel. The specific channel to be opened can be
   ' specified by setting any of the following properties:
   '     DeviceSerialNumber
   '     DeviceLabel
   '     Channel
   '     HubPort
   '     IsHubPortDevice
   '     ServerName
   '     IsLocal
   '     IsRemote
   ' Phidget_open() will return immediately, with the attachment process
   ' proceeding asynchronously.  Use the Attach event or Attached property
   ' to determine when the channel is ready to use.
   '
   Public Declare Function PhOpen& _
      Lib "Phidget22.dll" _
      Alias "Phidget_open" ( _
      ByVal handle&)

   ' =======================================================================
   ' Phidget_openWaitForAttachment(
   '     PhidgetHandle ch,
   '     uint32_t  timeout
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     timeout: Timeout in milliseconds
   ' Return: The Phidget return code
   ' Opens the Phidget channel and waits a defined amount of time for
   ' the device to attach.  The specific channel to be opened can be
   ' specified by setting any of the following properties:
   '     DeviceSerialNumber
   '     DeviceLabel
   '     Channel
   '     HubPort
   '     IsHubPortDevice
   '     ServerName
   '     IsLocal
   '     IsRemote
   ' Phidget_openWaitForAttachment() will block until the channel is attached
   ' or a timeout occurs.  A timeout value of 0 will wait forever.
   '
   Public Declare Function PhOpenWaitForAttachment& _
      Lib "Phidget22.dll" _
      Alias "Phidget_openWaitForAttachment" ( _
      ByVal handle&, _
      ByVal timeOutMs&)    ' unsigned integer

   ' =======================================================================
   ' Phidget_getParent(
   '     PhidgetHandle ch,
   '     PhidgetHandle *parent
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     parent: The handle of the parent
   ' Return: The Phidget return code
   ' Gets the handle of the parent device of the given Phidget handle.
   ' For example, this would refer to the device the channel is a part of,
   ' or the Hub that a device is plugged into.  This is useful when used
   ' alongside a Phidget Manager to create device trees like the one in
   ' the Phidget Control Panel. Phidget_release() must be called on the
   ' handle returned by this function to release the reference created
   ' by this call.  This can be used to travel up the device tree and
   ' get device information at each step.  The root device will return
   ' a null handle.  Parent handles always refer to devices.
   ' See Phidget_getIsChannel()
   '
   Public Declare Function PhGetParent& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getParent" ( _
      ByVal handle&, _
      ByRef parent&)

   ' =======================================================================
   ' Phidget_getServerHostname(
   '     PhidgetHandle ch,
   '     const char **serverHostname
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     serverHostname: The hostname of the channel's server
   ' Return: The Phidget return code
   ' Gets the hostname of the Phidget network server for network attached
   ' Phidgets.  Fails if channel is not connected to a Phidget network server.
   ' NOTE: After calling this function use:
   '       Str$ = SysAllocStringByteLen$(...StrPtr&, StrLen&(...StrPtr&))
   '
   Public Declare Function PhGetServerHostname& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getServerHostname" ( _
      ByVal handle&, _
      ByRef serverHostnameStrPtr&)

   ' =======================================================================
   ' Phidget_getServerName(
   '     PhidgetHandle ch,
   '     const char **serverName
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     serverName: The name of the Phidget network server the channel is from
   ' Return: The Phidget return code
   ' Gets the name of the Phidget network server the channel is attached to,
   ' if any.  Fails if the channel is not connected to a Phidget network server.
   ' NOTE: After calling this function use:
   '       Str$ = SysAllocStringByteLen$(...StrPtr&, StrLen&(...StrPtr&))
   Public Declare Function PhGetServerName& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getServerName" ( _
      ByVal handle&, _
      ByRef serverNameStrPtr&)

   ' =======================================================================
   ' Phidget_setServerName(
   '     PhidgetHandle ch,
   '     const char *serverName
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     serverName: The name of the Phidget network server the channel is from
   ' Return: The Phidget return code
   ' Specifies that this channel will be opened remotely, on a Phidget
   ' network server with this name.
   '
   Public Declare Function PhSetServerName& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setServerName" ( _
      ByVal handle&, _
      ByRef serverName$)

   ' =======================================================================
   ' Phidget_getServerPeerName(
   '     PhidgetHandle ch,
   '     const char **serverPeerName
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     serverPeerName: The address and port of the channel's server
   ' Return: The Phidget return code
   ' Gets the peer name (address and port) of the Phidget server for
   ' network attached Phidgets, formatted as: address:port
   ' Fails if the channel is not connected to a Phidget network server.
   ' NOTE: After calling this function use:
   '       Str$ = SysAllocStringByteLen$(...StrPtr&, StrLen&(...StrPtr&))
   Public Declare Function PhGetServerPeerName& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getServerPeerName" ( _
      ByVal handle&, _
      ByRef serverPeerNameStrPtr&)

   ' =======================================================================
   ' Phidget_writeDeviceLabel(
   '     PhidgetHandle ch,
   '     const char *deviceLabel
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     deviceLabel: The device label
   ' Return: The Phidget return code
   ' Writes a label to the device in the form of a string in the device
   ' flash memory.  This label can then be used to identify the device,
   ' and will persist across power cycles.  The label can be at most
   ' 10 UTF-16 code units.  Most unicode characters take up a single
   ' code unit, but some, such as emoji, can take several.  Some devices
   ' can not have their labels set from Windows.  For these devices the
   ' label should be set from Linux or macOS.
   ' See using a label for more information about how to use labels with Phidgets.
   ' Example:  stat& = PhWriteDeviceLabel&(handle&, StrPtr(deviceLabel$))
   ' Sets the label of a Phidget. Note that this is not supported on
   '     very old Phidgets, and not supported in Windows7.
   '
   Public Declare Function PhWriteDeviceLabel& _
      Lib "Phidget22.dll" _
      Alias "Phidget_writeDeviceLabel" ( _
      ByVal handle&, _
      ByRef Label$)

   ' =======================================================================
   ' Events:
   ' =======================================================================
   
   ' Phidget_setOnAttachHandler(
   '     PhidgetHandle ch,
   '     Phidget_OnAttachCallback handler,
   '     void *ctx
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     handler: The callback function  (AddressOf PhOnAttach)
   '     ctx: User provided context pointer
   ' Return: The Phidget return code
   ' Assigns a handler that will be called when the Attach event occurs
   '
   ' This function requires PhOnAttach& callback procedure
   ' handlerPtr& must be set to AddressOf PhOnAttach&
   '
   Public Declare Function PhSetOnAttach& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setOnAttachHandler" ( _
      ByVal handle&, _
      ByVal handlerPtr&, _
      ByRef userData&)
    
   ' =======================================================================
   ' Phidget_setOnDetachHandler(
   '     PhidgetHandle ch,
   '     Phidget_OnDetachCallback handler,
   '     void *ctx
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     handler: The callback function  (AddressOf PhOnDetach)
   '     ctx: User provided context pointer
   ' Return: The Phidget return code
   ' Assigns a handler that will be called when the Detach event occurs
   '
   ' This function requires PhOnDetach& callback procedure
   ' handlerPtr& must be set to AddressOf PhOnDetach&
   '
   Public Declare Function PhSetOnDetach& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setOnDetachHandler" ( _
      ByVal handle&, _
      ByVal handlerPtr&, _
      ByRef userData&)
  
   ' =======================================================================
   ' Phidget_setOnErrorHandler(
   '     PhidgetHandle ch,
   '     Phidget_OnErrorCallback handler,
   '     void *ctx
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     handler: The callback function  (AddressOf PhOnError)
   '     ctx: User provided context pointer
   ' Return: The Phidget return code
   ' Assigns a handler that will be called when the Error event occurs
   '
   ' This function requires PhOnError& callback procedure
   ' handlerPtr& must be set to AddressOf PhOnError&
   '
   Public Declare Function PhSetOnError& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setOnErrorHandler" ( _
      ByVal handle&, _
      ByVal handlerPtr&, _
      ByRef userData&)

   ' =======================================================================
   ' Phidget_setOnPropertyChangeHandler(
   '     PhidgetHandle ch,
   '     Phidget_OnPropertyChangeCallback handler,
   '     void *ctx
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     handler: The callback function  (AddressOf PhOnPropertyChange)
   '     ctx: User provided context pointer
   ' Return: The Phidget return code
   ' Assigns a handler that will be called when the PropertyChange event occurs
   '
   ' This function requires PhOnPropertyChange& callback procedure
   ' handlerPtr& must be set to AddressOf PhOnPropertyChange&
   '
   Public Declare Function PhSetOnPropertyChange& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setOnPropertyChangeHandler" ( _
      ByVal handle&, _
      ByVal handlerPtr&, _
      ByRef userData&)



'===========================================================================
'/* Additional functions */
'===========================================================================

   ' =======================================================================
   ' Frees a Phidget handle.
   ' phid - A closed phidget handle.
   ' Phidget_delete (PhidgetHandle *phid);
   '
   Public Declare Function PhDelete& _
      Lib "Phidget22.dll" _
      Alias "Phidget_delete" ( _
      ByRef handle&)

   ' =======================================================================
   ' Phidget_getChildDevices (
   '     PhidgetHandle phid,
   '     PhidgetHandle *arr,
   '     size_t *arrCnt);
   '
   Public Declare Function PhGetChildDevices& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getChildDevices" ( _
      ByVal handle&, _
      ByRef arr&, _
      ByRef arrCnt&)

   ' =======================================================================
   ' Phidget_releaseDevices (
   '     PhidgetHandle *arr,
   '     size_t arrCnt);
   '
   Public Declare Function PhReleaseDevices& _
      Lib "Phidget22.dll" _
      Alias "Phidget_releaseDevices" ( _
      ByRef arr&, _
      ByVal arrCnt&)


'===========================================================================
'/* Mesh device properties */
'===========================================================================
   
   ' =======================================================================
   ' Phidget_setMeshMode (
   '     PhidgetHandle phid,
   '     Phidget_MeshMode mode);
   '
   Public Declare Function PhSetMeshMode& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setMeshMode" ( _
      ByVal handle&, _
      ByVal mode&)

   ' =======================================================================
   ' Phidget_getMeshMode (
   '     PhidgetHandle phid,
   '     Phidget_MeshMode *mode);
   '
   Public Declare Function PhGetMeshMode& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getMeshMode" ( _
      ByVal handle&, _
      ByRef mode&)


'===========================================================================
' Event Callbacks, Subs, and Functions
'===========================================================================

'===========================================================================
Public Static Function PhOnAttach&(ByVal handle&, ByRef userData&)
   ' Phidget_OnAttachCallback void CCONV onAttach(
   '     PhidgetHandle ch,
   '     void *ctx
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     ctx: Pointer to the user context
   ' Occurs when the channel is attached to a physical channel on a Phidget.
   ' PhSetOnAttach must be called prior to calling Phidget_open(), and
   ' will be called when the Phidget library matches the channel with a
   ' physical channel on a Phidget.  PhOnAttach may be called more than
   ' once if the channel is detached during its lifetime.  PhOnAttach is
   ' the recommended place to configure properties of the channel such
   ' as the data interval or change trigger.
   '
   ' Sample code for startup:
   '     stat& = PhSetOnAttach&(handle&, AddressOf PhOnAttach, userData&)
   
   ' ========== User code HERE =========
   ' CAUTION: (Using CALLS here might cause .exe to crash)
   Form1.Label6(userData&) = "Attached"
   PhOnAttach& = 0& ' Return code
   ' ===================================

End Function
   
'===========================================================================
Public Static Function PhOnDetach&(ByVal handle&, ByRef userData&)
   ' Phidget_OnDetachCallback void CCONV onDetach(
   '     PhidgetHandle ch,
   '     void *ctx
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     ctx: Pointer to the user context
   ' Occurs when the channel is detached from a Phidget device channel.
   ' Phidget_OnDetachCallback typically occurs when
   ' the Phidget device is removed from the system.
   '
   ' Sample code for startup:
   '     stat& = PhSetOnDetach&(handle&, AddressOf PhOnDetach, userData&)
   
   ' ========== User code HERE =========
   ' CAUTION: (Using CALLS here might cause .exe to crash)
   ' Note:
   ' If I put to much code here, the program will crash on exit when
   ' running in interpeted mode after un-plugging and re-plugging the
   ' USB cable.  EXE still shuts down OK if I don't do any Calls.  ???
   Form1.Label6(userData&) = "Detached"
   Form1.Label7(userData&) = "---"
   PhOnDetach& = 0& ' Return code
   ' ===================================
   
   
End Function
   
'===========================================================================
Public Static Function PhOnError&(ByVal handle&, ByRef userData&, _
                            ByVal errCode&, ByRef description$)
   ' Phidget_OnErrorCallback void CCONV onError(
   '     PhidgetHandle ch,
   '     void *ctx,
   '     Phidget_ErrorEventCode code,
   '     const char *description
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     ctx: Pointer to the user context
   '     code: The error code
   '     description: The error description
   ' Phidget_OnErrorCallback is called when an error condition has been detected.
   ' See the documentation for your specific channel class to see what error
   ' events it might throw.
   '
   ' Sample code for startup:
   '     stat& = PhSetOnError&(handle&, AddressOf PhOnError, userData&)
   
   ' ========== User code HERE =========
   ' CAUTION: (Using CALLS here might cause .exe to crash)
   Call Form1.DisplayStatus(userData&, errCode&, description$)
   PhOnError& = 0& ' Return code
   ' ===================================

End Function
   
'===========================================================================
Public Static Function OnPropertyChange&(ByVal handle&, ByRef userData&, _
                              ByRef propertyName$)
   ' Phidget_OnPropertyChangeCallback void CCONV onPropertyChange(
   '     PhidgetHandle ch,
   '     void *ctx,
   '     const char *propertyName
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     ctx: Pointer to the user context
   '     propertyName: The name of the property that has changed
   ' Occurs when a property is changed externally from the user channel,
   ' usually from a network client attached to the same channel.
   '
   ' Sample code for startup:
   '     stat& = PhSetPropertyChange&(handle&, _
   '                AddressOf PhOnPropertyChange, userData&)
   
   ' ========== User code HERE =========
   ' CAUTION: (Using CALLS here might cause .exe to crash)
   OnPropertyChange& = 0& ' Return code
   ' ===================================
   
End Function
   
'===========================================================================
Public Static Function PhGetErrorDescription$(errCode&)
   ' Example:  errStr$ = PhGetErrorDescription&(errCode&)
   '
   ' For returning string data from the Phidget22 DLL.
   ' VB6 expects string data to be returned ByRef.  Which is a
   ' pointer to the actual string.  The Phidget22 DLL returns
   ' a pointer to a pointer which points to the actual data.
   ' The string is also NULL (0) terminated.  The "0" must be
   ' removed before VB can display the string correctly.
   ' Fortunately; There is an API function that takes care of
   ' all of this for us. "SysAllocStringByteLen"

   Dim stat&
   Dim cStrPtr&
   
   PhGetErrorDescription$ = "" ' Return with nothing if error
   stat& = Phidget_getErrorDescription&(errCode&, cStrPtr&) ' Get cStrPtr&
   If stat& = 0 Then
      ' Got pointer to string, now get string
      PhGetErrorDescription$ = SysAllocStringByteLen$(cStrPtr&, StrLen&(cStrPtr&))
   End If
   
End Function

   
   
   ' =======================================================================
   ' Enumerations:
   ' =======================================================================
    
   ' PhidgetReturnCode - The result codes returned by all functions
   '
   ' EPHIDGET_OK 0x0 Success
   ' EPHIDGET_PERM 0x1 Not Permitted
   ' EPHIDGET_NOENT 0x2 No Such Entity
   ' EPHIDGET_TIMEOUT 0x3 Timed Out
   ' EPHIDGET_KEEPALIVE 0x3a Keep Alive Failure
   ' EPHIDGET_INTERRUPTED 0x4 Op Interrupted
   ' EPHIDGET_IO 0x5 IO Issue
   ' EPHIDGET_NOMEMORY 0x6 Memory Issue
   ' EPHIDGET_ACCESS 0x7 Access (Permission) Issue
   ' EPHIDGET_FAULT 0x8 Address Issue
   ' EPHIDGET_BUSY 0x9 Resource Busy
   ' EPHIDGET_EXIST 0xa Object Exists
   ' EPHIDGET_NOTDIR 0xb Object is not a directory
   ' EPHIDGET_ISDIR 0xc Object is a directory
   ' EPHIDGET_INVALID 0xd Invalid
   ' EPHIDGET_NFILE 0xe Too many open files in system
   ' EPHIDGET_MFILE 0xf Too many open files
   ' EPHIDGET_NOSPC 0x10 Not enough space
   ' EPHIDGET_FBIG 0x11 File too Big
   ' EPHIDGET_ROFS 0x12 Read Only Filesystem
   ' EPHIDGET_RO 0x13 Read Only Object
   ' EPHIDGET_UNSUPPORTED 0x14 Operation Not Supported
   ' EPHIDGET_INVALIDARG 0x15 Invalid Argument
   ' EPHIDGET_AGAIN 0x16 Try again
   ' EPHIDGET_NOTEMPTY 0x1a No Empty
   ' EPHIDGET_UNEXPECTED 0x1c Unexpected Error
   ' EPHIDGET_DUPLICATE 0x1b Duplicate
   ' EPHIDGET_BADPASSWORD 0x25 Bad Credential
   ' EPHIDGET_NETUNAVAIL 0x2d Network Unavailable
   ' EPHIDGET_CONNREF 0x23 Connection Refused
   ' EPHIDGET_CONNRESET 0x2e Connection Reset
   ' EPHIDGET_HOSTUNREACH 0x30 No route to host
   ' EPHIDGET_NODEV 0x28 No Such Device
   ' EPHIDGET_WRONGDEVICE 0x32 Wrong Device
   ' EPHIDGET_PIPE 0x29 Broken Pipe
   ' EPHIDGET_RESOLV 0x2c Name Resolution Failure
   ' EPHIDGET_UNKNOWNVAL 0x33 Unknown or Invalid Value
   ' EPHIDGET_NOTATTACHED 0x34 Device not Attached
   ' EPHIDGET_INVALIDPACKET 0x35 Invalid or Unexpected Packet
   ' EPHIDGET_2BIG 0x36 Argument List Too Long
   ' EPHIDGET_BADVERSION 0x37 Bad Version
   ' EPHIDGET_CLOSED 0x38 Closed
   ' EPHIDGET_NOTCONFIGURED 0x39 Not Configured
   ' EPHIDGET_EOF 0x1f End of File

   ' Phidget_ErrorEventCode - The error code from an Error event
   '
   ' EEPHIDGET_BADVERSION 0x1 Client and Server protocol versions don't match.
   ' EEPHIDGET_BUSY 0x2 Phidget is in use.
   ' EEPHIDGET_NETWORK 0x3 Networking communication error.
   ' EEPHIDGET_DISPATCH 0x4 An error occured dispatching a command or event.
   ' EEPHIDGET_FAILURE 0x5 A general failure occured - see description for details.
   ' EEPHIDGET_OK 0x1000 An error state has cleared.
   ' EEPHIDGET_OVERRUN 0x1002 A sampling overrun happened in firmware.
   ' EEPHIDGET_PACKETLOST 0x1003 One or more packets were lost.
   ' EEPHIDGET_WRAP 0x1004 A variable has wrapped around.
   ' EEPHIDGET_OVERTEMP 0x1005 Over-temperature condition detected.
   ' EEPHIDGET_OVERCURRENT 0x1006 Over-current condition detected.
   ' EEPHIDGET_OUTOFRANGE 0x1007 Out of range condition detected.
   ' EEPHIDGET_BADPOWER 0x1008 Power supply problem detected.
   ' EEPHIDGET_SATURATION 0x1009 Saturation condition detected.
   ' EEPHIDGET_OVERVOLTAGE 0x100b Over-voltage condition detected.
   ' EEPHIDGET_FAILSAFE 0x100c Fail-safe condition detected.
   ' EEPHIDGET_VOLTAGEERROR 0x100d Voltage error detected.
   ' EEPHIDGET_ENERGYDUMP 0x100e Energy dump condition detected.
   ' EEPHIDGET_MOTORSTALL 0x100f Motor stall detected.

   ' Phidget_DeviceID
   '
   ' PHIDID_NOTHING 0x0 Unknown device
   ' PHIDID_INTERFACEKIT_4_8_8 0x1 PhidgetInterfaceKit 4/8/8
   ' PHIDID_1000 0x2 PhidgetServo 1-Motor (1000)
   ' PHIDID_1001 0x3 PhidgetServo 4-Motor (1001)
   ' PHIDID_1002 0x4 PhidgetAnalog 4-Output (1002)
   ' PHIDID_1008 0x5 PhidgetAccelerometer 2-Axis (1008)
   ' PHIDID_1010_1013_1018_1019 0x6 PhidgetInterfaceKit 8/8/8 (1010, 1013, 1018, 1019)
   ' PHIDID_1011 0x7 PhidgetInterfaceKit 2/2/2 (1011)
   ' PHIDID_1012 0x8 PhidgetInterfaceKit 0/16/16 (1012)
   ' PHIDID_1014 0x9 PhidgetInterfaceKit 0/0/4 (1014)
   ' PHIDID_1015 0xa PhidgetLinearTouch (1015)
   ' PHIDID_1016 0xb PhidgetCircularTouch (1016)
   ' PHIDID_1017 0xc PhidgetInterfaceKit 0/0/8 (1017)
   ' PHIDID_1023 0xd PhidgetRFID (1023)
   ' PHIDID_1024 0xe PhidgetRFID Read-Write (1024)
   ' PHIDID_1030 0xf PhidgetLED-64 (1030)
   ' PHIDID_1031 0x10 PhidgetLED-64 Advanced (1031)
   ' PHIDID_1032 0x11 PhidgetLED-64 Advanced (1032)
   ' PHIDID_1040 0x12 PhidgetGPS (1040)
   ' PHIDID_1041 0x13 PhidgetSpatial 0/0/3 Basic (1041)
   ' PHIDID_1042 0x14 PhidgetSpatial 3/3/3 Basic (1042)
   ' PHIDID_1043 0x15 PhidgetSpatial Precision 0/0/3 High Resolution (1043)
   ' PHIDID_1044 0x16 PhidgetSpatial Precision 3/3/3 High Resolution (1044)
   ' PHIDID_1045 0x17 PhidgetTemperatureSensor IR (1045)
   ' PHIDID_1046 0x18 PhidgetBridge 4-Input (1046)
   ' PHIDID_1047 0x19 PhidgetEncoder HighSpeed 4-Input (1047)
   ' PHIDID_1048 0x1a PhidgetTemperatureSensor 4-input (1048)
   ' PHIDID_1049 0x1b PhidgetSpatial 0/0/3 (1049)
   ' PHIDID_1051 0x1c PhidgetTemperatureSensor 1-Input (1051)
   ' PHIDID_1052 0x1d PhidgetEncoder Mechanical (1052)
   ' PHIDID_1053 0x1e PhidgetAccelerometer 2-Axis (1053)
   ' PHIDID_1054 0x1f PhidgetFrequencyCounter (1054)
   ' PHIDID_1055 0x20 PhidgetIR (1055)
   ' PHIDID_1056 0x21 PhidgetSpatial 3/3/3 (1056)
   ' PHIDID_1057 0x22 PhidgetEncoder HighSpeed (1057)
   ' PHIDID_1058 0x23 PhidgetPHSensor (1058)
   ' PHIDID_1059 0x24 PhidgetAccelerometer 3-Axis (1059)
   ' PHIDID_1060 0x25 PhidgetMotorControl LV (1060)
   ' PHIDID_1061 0x26 PhidgetAdvancedServo 8-Motor (1061)
   ' PHIDID_1062 0x27 PhidgetStepper Unipolar 4-Motor (1062)
   ' PHIDID_1063 0x28 PhidgetStepper Bipolar 1-Motor (1063)
   ' PHIDID_1064 0x29 PhidgetMotorControl HC (1064)
   ' PHIDID_1065 0x2a PhidgetMotorControl 1-Motor (1065)
   ' PHIDID_1066 0x2b PhidgetAdvancedServo 1-Motor (1066)
   ' PHIDID_1067 0x2c PhidgetStepper Bipolar HC (1067)
   ' PHIDID_1202_1203 0x2d
   ' PhidgetTextLCD 20x2 with PhidgetInterfaceKit 8/8/8 (1201, 1202, 1203)
   ' PHIDID_1204 0x2e PhidgetTextLCD Adapter (1204)
   ' PHIDID_1215__1218 0x2f PhidgetTextLCD 20x2 (1215/1216/1217/1218)
   ' PHIDID_1219__1222 0x30
   ' PhidgetTextLCD 20x2 with PhidgetInterfaceKit 0/8/8 (1219, 1220, 1221, 1222)
   ' PHIDID_ADP1000 0x31 pH Adapter
   ' PHIDID_DAQ1000 0x33 Analog Input Module x8
   ' PHIDID_DAQ1200 0x34 Digital Input 4
   ' PHIDID_DAQ1300 0x35 Digital Input 4 Isolated
   ' PHIDID_DAQ1301 0x36 Digital Input 16
   ' PHIDID_DAQ1400 0x37 Versatile Input
   ' PHIDID_DAQ1500 0x38 Bridge
   ' PHIDID_DCC1000 0x39 DC Motor Controller with PID
   ' PHIDID_DST1000 0x3a 200mm Distance Sensor
   ' PHIDID_DST1200 0x3b Sonar Distance Sensor
   ' PHIDID_ENC1000 0x3c Encoder
   ' PHIDID_HIN1000 0x3d Capacitive Touch Sensor
   ' PHIDID_HIN1001 0x3e Capacitive Scroll
   ' PHIDID_HIN1100 0x3f Joystick
   ' PHIDID_HUB0000 0x40 Phidget Hub with 6 ports
   ' PHIDID_HUB0004 0x43 Phidget SPI VINT Hub with 6 ports
   ' PHIDID_HUM1000 0x45 Humidity Sensor
   ' PHIDID_LCD1100 0x46 LCD
   ' PHIDID_LED1000 0x47 LED Driver 32
   ' PHIDID_LUX1000 0x48 Light Sensor
   ' PHIDID_MOT1100 0x49 Accelerometer 0/0/3
   ' PHIDID_MOT1101 0x4a Spatial 3/3/3
   ' PHIDID_OUT1000 0x4b Analog Output 0-5V
   ' PHIDID_OUT1001 0x4c Analog Output (+/-)10V
   ' PHIDID_OUT1002 0x4d Analog Output (+/-)10V - 16 bit
   ' PHIDID_OUT1100 0x4e Digital Output 4
   ' PHIDID_PRE1000 0x4f Barometer
   ' PHIDID_RCC1000 0x50 8-Servo Controller
   ' PHIDID_REL1000 0x51 Power Relay 4
   ' PHIDID_REL1100 0x52 Digital Output 4 Isolated
   ' PHIDID_REL1101 0x53 Digital Output 16 Isolated
   ' PHIDID_SAF1000 0x54 Power Supply Protector
   ' PHIDID_SND1000 0x55 Sound Pressure Level Sensor
   ' PHIDID_STC1000 0x56 Bipolar Stepper Motor Controller
   ' PHIDID_TMP1000 0x57 Integrated Temperature Sensor
   ' PHIDID_TMP1100 0x58 Thermocouple 1
   ' PHIDID_TMP1101 0x59 Thermocouple 4
   ' PHIDID_TMP1200 0x5a RTD
   ' PHIDID_VCP1000 0x5c Voltage Sensor High Precision
   ' PHIDID_VCP1001 0x5d Voltage Sensor Large
   ' PHIDID_VCP1002 0x5e Voltage Sensor Small
   ' PHIDID_DIGITALINPUT_PORT 0x5f Hub Port in Digital Input mode
   ' PHIDID_DIGITALOUTPUT_PORT 0x60 Hub Port in Digital Output mode
   ' PHIDID_VOLTAGEINPUT_PORT 0x61 Hub Port in Voltage Input mode
   ' PHIDID_VOLTAGERATIOINPUT_PORT 0x62 Hub Port in Voltage Ratio Input mode
   ' PHIDID_VCP1100 0x69 30A Current Sensor
   ' PHIDID_DCC1100 0x6c BLDC Motor Controller
   ' PHIDID_HIN1101 0x6d Dial Encoder
   ' PHIDID_DCC1001 0x6e Small DC Motor Controller
   ' PHIDID_DICTIONARY 0x6f Dictionary
   ' PHIDID_STC1001 0x73 Bipolar Stepper Motor SmallController Phidg
   '


   ' Phidget device class
   '
   ' PHIDCLASS_ACCELEROMETER 0x1 PhidgetAccelerometer device
   ' PHIDCLASS_ADVANCEDSERVO 0x2 PhidgetAdvancedServo device
   ' PHIDCLASS_ANALOG 0x3 PhidgetAnalog device
   ' PHIDCLASS_BRIDGE 0x4 PhidgetBridge device
   ' PHIDCLASS_ENCODER 0x5 PhidgetEncoder device
   ' PHIDCLASS_FREQUENCYCOUNTER 0x6 PhidgetFrequencyCounter device
   ' PHIDCLASS_GPS 0x7 PhidgetGPS device
   ' PHIDCLASS_HUB 0x8 Phidget VINT Hub device
   ' PHIDCLASS_INTERFACEKIT 0x9 PhidgetInterfaceKit device
   ' PHIDCLASS_IR 0xa PhidgetIR device
   ' PHIDCLASS_LED 0xb PhidgetLED device
   ' PHIDCLASS_MOTORCONTROL 0xd PhidgetMotorControl device
   ' PHIDCLASS_PHSENSOR 0xe PhidgetPHSensor device
   ' PHIDCLASS_RFID 0xf PhidgetRFID device
   ' PHIDCLASS_SERVO 0x10 PhidgetServo device
   ' PHIDCLASS_SPATIAL 0x11 PhidgetSpatial device
   ' PHIDCLASS_STEPPER 0x12 PhidgetStepper device
   ' PHIDCLASS_TEMPERATURESENSOR 0x13 PhidgetTemperatureSensor device
   ' PHIDCLASS_TEXTLCD 0x14 PhidgetTextLCD device
   ' PHIDCLASS_VINT 0x15 Phidget VINT device
   ' PHIDCLASS_DICTIONARY 0x18 Dictionary device
   '
   ' Phidget_ChannelClass
   ' PHIDCHCLASS_ACCELEROMETER 0x1 Accelerometer channel
   ' PHIDCHCLASS_CURRENTINPUT 0x2 Current input channel
   ' PHIDCHCLASS_DCMOTOR 0x4 DC motor channel
   ' PHIDCHCLASS_DIGITALINPUT 0x5 Digital input channel
   ' PHIDCHCLASS_DIGITALOUTPUT 0x6 Digital output channel
   ' PHIDCHCLASS_DISTANCESENSOR 0x7 Distance sensor channel
   ' PHIDCHCLASS_ENCODER 0x8 Encoder channel
   ' PHIDCHCLASS_FREQUENCYCOUNTER 0x9 Frequency counter channel
   ' PHIDCHCLASS_GPS 0xa GPS channel
   ' PHIDCHCLASS_LCD 0xb LCD channel
   ' PHIDCHCLASS_GYROSCOPE 0xc Gyroscope channel
   ' PHIDCHCLASS_HUB 0xd VINT Hub channel
   ' PHIDCHCLASS_CAPACITIVETOUCH 0xe Capacitive Touch channel
   ' PHIDCHCLASS_HUMIDITYSENSOR 0xf Humidity sensor channel
   ' PHIDCHCLASS_IR 0x10 IR channel
   ' PHIDCHCLASS_LIGHTSENSOR 0x11 Light sensor channel
   ' PHIDCHCLASS_MAGNETOMETER 0x12 Magnetometer channel
   ' PHIDCHCLASS_PHSENSOR 0x25 pH sensor channel
   ' PHIDCHCLASS_POWERGUARD 0x14 Power guard channel
   ' PHIDCHCLASS_PRESSURESENSOR 0x15 Pressure sensor channel
   ' PHIDCHCLASS_RCSERVO 0x16 RC Servo channel
   ' PHIDCHCLASS_RESISTANCEINPUT 0x17 Resistance input channel
   ' PHIDCHCLASS_RFID 0x18 RFID channel
   ' PHIDCHCLASS_SOUNDSENSOR 0x19 Sound sensor channel
   ' PHIDCHCLASS_SPATIAL 0x1a Spatial channel
   ' PHIDCHCLASS_STEPPER 0x1b Stepper channel
   ' PHIDCHCLASS_TEMPERATURESENSOR 0x1c Temperature sensor channel
   ' PHIDCHCLASS_VOLTAGEINPUT 0x1d Voltage input channel
   ' PHIDCHCLASS_VOLTAGEOUTPUT 0x1e Voltage output channel
   ' PHIDCHCLASS_VOLTAGERATIOINPUT 0x1f Voltage ratio input channel
   ' PHIDCHCLASS_MOTORPOSITIONCONTROLLER 0x22 Motor position control channel.
   ' PHIDCHCLASS_BLDCMOTOR 0x23 BLDC motor channel
   ' PHIDCHCLASS_DICTIONARY 0x24 Dictionary

   ' Phidget channel sub class
   '
   ' PHIDCHSUBCLASS_NONE 0x1 No subclass
   ' PHIDCHSUBCLASS_DIGITALOUTPUT_DUTY_CYCLE 0x10 Digital output duty cycle
   ' PHIDCHSUBCLASS_DIGITALOUTPUT_LED_DRIVER 0x11 Digital output LED driver
   ' PHIDCHSUBCLASS_TEMPERATURESENSOR_RTD 0x20 Temperature sensor RTD
   ' PHIDCHSUBCLASS_TEMPERATURESENSOR_THERMOCOUPLE 0x21 Temperature sensor thermocouple
   ' PHIDCHSUBCLASS_VOLTAGEINPUT_SENSOR_PORT 0x30 Voltage sensor port
   ' PHIDCHSUBCLASS_VOLTAGERATIOINPUT_SENSOR_PORT 0x40 Voltage ratio sensor port
   ' PHIDCHSUBCLASS_VOLTAGERATIOINPUT_BRIDGE 0x41 Voltage ratio bridge input
   ' PHIDCHSUBCLASS_LCD_GRAPHIC 0x50 Graphic LCD
   ' PHIDCHSUBCLASS_LCD_TEXT 0x51 Text LCD
   ' PHIDCHSUBCLASS_ENCODER_MODE_SETTABLE 0x60 Encoder IO mode settable
   ' PHIDCHSUBCLASS_RCSERVO_EMBEDDED 0x70 RC Servo Embedded
   
   ' Constants:
   '
   ' PHIDGET_SERIALNUMBER_ANY    -1
   '     Pass to Phidget_setDeviceSerialNumber() to open any serial number.
   ' PHIDGET_HUBPORT_ANY         -1
   '     Pass to Phidget_setHubPort() to open any hub port.
   ' PHIDGET_CHANNEL_ANY         -1
   '      Pass to Phidget_setChannel() to open any channel.
   ' PHIDGET_LABEL_ANY
   '     NULL Pass to Phidget_setDeviceLabel() to open any label.
   ' PHIDGET_TIMEOUT_INFINITE    0
   '     Pass to Phidget_openWaitForAttachment() for an infinite timeout.
   ' PHIDGET_TIMEOUT_DEFAULT     500
   '     Pass to Phidget_openWaitForAttachment() for the default timeout.
   '
